<?php
/**
 * @author Silvan Engeler <silvan.engeler@threema.ch>
 * @copyright Copyright (c) 2020 Threema GmbH
 * @link https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use mysql_xdevapi\Exception;
use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Helpers\E2EHelper;
use Threema\MsgApi\PublicKeyStore;

class GroupCreate extends Base {
	const argMembers = 'members';
	/**
	 * @var PublicKeyStore
	 */
	private $publicKeyStore;

	/**
	 * @param PublicKeyStore $publicKeyStore
	 */
	function __construct(PublicKeyStore $publicKeyStore) {
		parent::__construct('Send a End-to-End Encrypted Video Message',
			[self::argThreemaId, self::argFrom, self::argSecret, self::argPrivateKey, self::argGroupId],
			'Send a group create to the given ID (members are comma-separated group members, no members = remove group)',
			[self::argMembers]);
		$this->publicKeyStore = $publicKeyStore;
	}

	function doRun() {
		$threemaId = $this->getArgument(self::argThreemaId);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);
		$groupId = trim($this->getArgument(self::argGroupId));

		if (strlen($groupId) != 8)
		{
			throw new Exception('invalid group id '.$groupId);
		}

		$temporaryMembers = explode(',', $this->getArgument(self::argMembers));
		$members = [$from];
		foreach($temporaryMembers as $memberId)
		{
			if (strlen($memberId) > 0)
			{
				$members[] = Common::verifyThreemaId($memberId);
			}
		}

		Common::required($threemaId, $from, $secret, $privateKey);

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);
		$helper = new E2EHelper($privateKey, $connector);

		$result = $helper->sendGroupCreate(
			$threemaId,
			$groupId,
			$members
		);

		if($result->isSuccess()) {
			Common::l('Message ID: '.$result->getMessageId());
		}
		else {
			Common::e('Error: '.$result->getErrorMessage());
		}
	}
}
