<?php
/**
 * @author Silvan Engeler <silvan.engeler@threema.ch>
 * @copyright Copyright (c) 2020 Threema GmbH
 * @link https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Helpers\E2EHelper;
use Threema\MsgApi\PublicKeyStore;

class GroupSendE2ELocation extends Base {
	const argLatitude = 'latitude';
	const argLongitude = 'longitude';
	const argAccuracy = 'accuracy';
	const argPOIName = 'poiName';
	const argPOIAddress = 'poiAddress';

	/**
	 * @var PublicKeyStore
	 */
	private $publicKeyStore;

	/**
	 * @param PublicKeyStore $publicKeyStore
	 */
	function __construct(PublicKeyStore $publicKeyStore) {
		parent::__construct('Send End-to-End Encrypted Location Message to a group',
			[self::argThreemaId, self::argFrom, self::argSecret, self::argPrivateKey, self::argGroupId, self::argLatitude, self::argLongitude],
			'Encrypt a group location and send it to the given given ID. \'from\' is the API identity and \'secret\' is the API secret. Prints the message ID on success.',
			[self::argAccuracy, self::argPOIName, self::argPOIAddress]);
		$this->publicKeyStore = $publicKeyStore;
	}

	function doRun() {
		$threemaId = $this->getArgumentThreemaId(self::argThreemaId);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);
		$groupId = $this->getArgumentGroupId(self::argGroupId);
		$latitude = (float) $this->getArgument(self::argLatitude);
		$longitude = (float) $this->getArgument(self::argLongitude);
		$accuracy = (float) $this->getArgument(self::argAccuracy);
		$poiName = $this->getArgument(self::argPOIName);
		$poiAddress = $this->getArgument(self::argPOIAddress);

		Common::required($threemaId, $from, $secret, $privateKey, $groupId, $latitude, $longitude);

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);

		$helper = new E2EHelper($privateKey, $connector);
		$result = $helper->sendGroupLocationMessage($groupId, $threemaId, $latitude, $longitude, $accuracy, $poiName, $poiAddress);

		if($result->isSuccess()) {
			Common::l('Message ID: '.$result->getMessageId());
		}
		else {
			Common::e('Error: '.$result->getErrorMessage());
		}
	}
}
