<?php
/**
 * @author Silvan Engeler <silvan.engeler@threema.ch>
 * @copyright Copyright (c) 2020 Threema GmbH
 * @link https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use mysql_xdevapi\Exception;
use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Helpers\E2EHelper;
use Threema\MsgApi\PublicKeyStore;

class GroupSetPhoto extends Base {
	const argPhotoPath = 'photo';

	/**
	 * @var PublicKeyStore
	 */
	private $publicKeyStore;

	/**
	 * @param PublicKeyStore $publicKeyStore
	 */
	function __construct(PublicKeyStore $publicKeyStore) {
		parent::__construct('Set a group photo',
			[self::argThreemaId, self::argFrom, self::argSecret, self::argPrivateKey, self::argGroupId, self::argPhotoPath],
			'Send group avatar');
		$this->publicKeyStore = $publicKeyStore;
	}

	function doRun() {
		$threemaId = $this->getArgument(self::argThreemaId);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);
		$groupId = trim($this->getArgument(self::argGroupId));
		$photoPath = $this->getArgumentFile(self::argPhotoPath);

		if (strlen($groupId) != 8)
		{
			throw new Exception('invalid group id '.$groupId);
		}

		Common::required($threemaId, $from, $secret, $privateKey, $photoPath);

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);
		$helper = new E2EHelper($privateKey, $connector);

		$result = $helper->sendGroupPhoto(
			$threemaId,
			$groupId,
			file_get_contents($photoPath)
		);

		if($result->isSuccess()) {
			Common::l('Message ID: '.$result->getMessageId());
		}
		else {
			Common::e('Error: '.$result->getErrorMessage());
		}
	}
}
