<?php
/**
 * @author Silvan Engeler <silvan.engeler@threema.ch>
 * @copyright Copyright (c) 2020 Threema GmbH
 * @link https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\PublicKeyStore;

class LookupIdBulk extends Base {
	const argEmailAddresses = 'emailAddresses';
	const argsPhoneNumbers = 'phoneNumbers';
	/**
	 * @var PublicKeyStore
	 */
	private $publicKeyStore;

	/**
	 * @param PublicKeyStore $publicKeyStore
	 */
	function __construct(PublicKeyStore $publicKeyStore) {
		parent::__construct('Bulk ID-Lookup By Email Addresses And Phone Numbers',
			array(self::argFrom, self::argSecret, self::argEmailAddresses, self::argsPhoneNumbers),
			'Lookup IDs linked to the given email or phone address (will be hashed locally).');
		$this->publicKeyStore = $publicKeyStore;
	}

	function doRun() {
		$from = $this->getArgumentThreemaId(self::argFrom);
		$secret = $this->getArgument(self::argSecret);

		$emailAddresses = explode(',', $this->getArgument(self::argEmailAddresses));
		$phoneNumbers = explode(',', $this->getArgument(self::argsPhoneNumbers));

		Common::required($from, $secret);

		//define connection settings
		$settings = new ConnectionSettings($from, $secret);

		//create a connection
		$connector = new Connection($settings, $this->publicKeyStore);

		$result = $connector->keyLookupBulk($emailAddresses, $phoneNumbers, false);
		Common::required($result);
		if($result->isSuccess()) {

			foreach($result->getResult() as $foundItem) {
				var_dump($foundItem);
			}
		}
		else {
			Common::e($result->getErrorMessage());
		}
	}
}
