<?php
/**
 * @author Silvan Engeler <silvan.engeler@threema.ch>
 * @copyright Copyright (c) 2020 Threema GmbH
 * @link https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Helpers\E2EHelper;
use Threema\MsgApi\PublicKeyStore;

class SendE2EAudio extends Base {
	const argDuration = 'duration';
	const argAudioFile = 'audioFILE';

	/**
	 * @var PublicKeyStore
	 */
	private $publicKeyStore;

	/**
	 * @param PublicKeyStore $publicKeyStore
	 */
	function __construct(PublicKeyStore $publicKeyStore) {
		parent::__construct('Send a End-to-End Encrypted Video Message',
			array(self::argThreemaId, self::argFrom, self::argSecret, self::argPrivateKey, self::argDuration, self::argAudioFile),
			'Encrypt the audio file and send the audio message to the given ID. \'from\' is the API identity and \'secret\' is the API secret. Prints the message ID on success.');
		$this->publicKeyStore = $publicKeyStore;
	}

	function doRun() {
		$threemaId = $this->getArgument(self::argThreemaId);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);

		$audioPath = $this->getArgumentFile(self::argAudioFile);
		$duration = $this->getArgument(self::argDuration);

		Common::required($threemaId, $from, $secret, $privateKey, $audioPath, $duration);

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);

		$helper = new E2EHelper($privateKey, $connector);
		$result = $helper->sendAudioMessage(
			$threemaId,
			$duration,
			$audioPath
		);

		if($result->isSuccess()) {
			Common::l('Message ID: '.$result->getMessageId());
		}
		else {
			Common::e('Error: '.$result->getErrorMessage());
		}
	}
}
